/*
FreeRTOS.org V4.1.2 - copyright (C) 2003-2006 Richard Barry.

This file is part of the FreeRTOS.org distribution.

FreeRTOS.org is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

FreeRTOS.org is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with FreeRTOS.org; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

A special exception to the GPL can be applied should you wish to distribute
a combined work that includes FreeRTOS.org, without being obliged to provide
the source code for any proprietary components.  See the licensing section
of http://www.FreeRTOS.org for full details of how and when the exception
can be applied.

***************************************************************************
See http://www.FreeRTOS.org for documentation, latest information, license
and contact details.  Please ensure to read the configuration and relevant
port sections of the online documentation.
***************************************************************************
*/

/* 
NOTE : Tasks run in system mode and the scheduler runs in Supervisor mode.
The processor MUST be in supervisor mode when vTaskStartScheduler is 
called.  The demo applications included in the FreeRTOS.org download switch
to supervisor mode prior to main being called.  If you are not using one of
these demo application projects then ensure Supervisor mode is used.
*/


/*
 * Creates all the application tasks, then starts the scheduler.
 *
 * A task defined by the function vBasicWEBServer is created.  This executes 
 * the lwIP stack and basic WEB server sample.  A task defined by the function
 * vUSBCDCTask.  This executes the USB to serial CDC example.  All the other 
 * tasks are from the set of standard demo tasks.  The WEB documentation 
 * provides more details of the standard demo application tasks.
 *
 * Main.c also creates a task called "Check".  This only executes every three
 * seconds but has the highest priority so is guaranteed to get processor time.
 * Its main function is to check the status of all the other demo application
 * tasks.  LED mainCHECK_LED is toggled every three seconds by the check task
 * should no error conditions be detected in any of the standard demo tasks.
 * The toggle rate increasing to 500ms indicates that at least one error has
 * been detected.
 *
 * Main.c includes an idle hook function that simply periodically sends data
 * to the USB task for transmission.
 */

/*
Changes from V3.2.2

+ Modified the stack sizes used by some tasks to permit use of the 
command line GCC tools.
*/

/* Library includes. */
#include <string.h>
#include <stdio.h>

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "task.h"

/* Demo application includes. */
#include "partest.h"
#include "PollQ.h"
#include "comtest2.h"
#include "semtest.h"
#include "flash.h"
#include "integer.h"
#include "BlockQ.h"
#include "BasicWEB.h"
#include "serial.h"

/* lwIP includes. */
#include "lwip/api.h" 

/* Hardware specific headers. */
#include "Board.h"
#include "AT91SAM9260.h"

/* Constants for the ComTest tasks. */
#define mainCOM_TEST_BAUD_RATE      ( ( unsigned portLONG ) 115200 )
#define mainCOM_TEST_LED            ( 5 )

/* Priorities/stacks for the various tasks within the demo application. */
#define mainQUEUE_POLL_PRIORITY     ( tskIDLE_PRIORITY + 1 )
#define mainCHECK_TASK_PRIORITY     ( tskIDLE_PRIORITY + 4 )
#define mainSEM_TEST_PRIORITY       ( tskIDLE_PRIORITY + 1 )
#define mainFLASH_PRIORITY          ( tskIDLE_PRIORITY + 2 )
#define mainBLOCK_Q_PRIORITY        ( tskIDLE_PRIORITY + 1 )
#define mainWEBSERVER_PRIORITY      ( tskIDLE_PRIORITY + 2 )
#define mainCOM_TEST_PRIORITY       ( tskIDLE_PRIORITY + 3 )

/* The rate at which the on board LED will toggle when there is/is not an
error. */
#define mainNO_ERROR_FLASH_PERIOD   ( ( portTickType ) 3000 / portTICK_RATE_MS  )
#define mainERROR_FLASH_PERIOD      ( ( portTickType ) 500 / portTICK_RATE_MS  )


/* The LED used by the check task to indicate the system status. */
#define mainCHECK_LED               ( 3 )
/*-----------------------------------------------------------*/

/*
 * Checks that all the demo application tasks are still executing without error
 * - as described at the top of the file.
 */
static portLONG prvCheckOtherTasksAreStillRunning( void );

/*
 * The task that executes at the highest priority and calls
 * prvCheckOtherTasksAreStillRunning().  See the description at the top
 * of the file.
 */
static void vErrorChecks( void *pvParameters );

/*
 * Configure the processor for use with the Atmel demo board.  This is very
 * minimal as most of the setup is performed in the startup code.
 */
static void prvSetupHardware( void );

/*-----------------------------------------------------------*/

/*
 * Setup hardware then start all the demo application tasks.
 */
int main( void )
{
  /* Setup the ports. */
  prvSetupHardware();

  /* Setup the IO required for the LED's. */
  vParTestInitialise();

  /* Setup lwIP. */
  vlwIPInit();

  /* Create the lwIP task.  This uses the lwIP RTOS abstraction layer.*/
  sys_thread_new( vBasicWEBServer, ( void * ) NULL, mainWEBSERVER_PRIORITY );

  /* Create the standard demo application tasks. */
  vStartPolledQueueTasks( mainQUEUE_POLL_PRIORITY );
  vStartSemaphoreTasks( mainSEM_TEST_PRIORITY );
  vStartLEDFlashTasks( mainFLASH_PRIORITY );
  vStartIntegerMathTasks( tskIDLE_PRIORITY );
  vStartBlockingQueueTasks( mainBLOCK_Q_PRIORITY );
  vAltStartComTestTasks( mainCOM_TEST_PRIORITY, mainCOM_TEST_BAUD_RATE, mainCOM_TEST_LED );

  /* Start the check task - which is defined in this file. */ 
  xTaskCreate( vErrorChecks, ( signed portCHAR * ) "Check", configMINIMAL_STACK_SIZE, NULL, mainCHECK_TASK_PRIORITY, NULL );

  /* Finally, start the scheduler. 

  NOTE : Tasks run in system mode and the scheduler runs in Supervisor mode.
  The processor MUST be in supervisor mode when vTaskStartScheduler is 
  called.  The demo applications included in the FreeRTOS.org download switch
  to supervisor mode prior to main being called.  If you are not using one of
  these demo application projects then ensure Supervisor mode is used here. */
  vTaskStartScheduler();

  /* Should never get here! */
  return 0;
}
/*-----------------------------------------------------------*/


static void prvSetupHardware( void )
{
  /* When using the JTAG debugger the hardware is not always initialised to
  the correct default state.  This line just ensures that this does not
  cause all interrupts to be masked at the start. */
  AT91C_BASE_AIC->AIC_EOICR = 0;

  /* Configure the PIO Lines corresponding to the LEDs to be outputs. */
  AT91C_BASE_PIOA->PIO_PER = LED_MASK; // Set in PIO mode
  AT91C_BASE_PIOA->PIO_OER = LED_MASK; // Configure in Output

  /* Enable the peripheral clocks. */
  AT91C_BASE_PMC->PMC_PCER = 1 << AT91C_ID_PIOA;
  AT91C_BASE_PMC->PMC_PCER = 1 << AT91C_ID_PIOB;
}
/*-----------------------------------------------------------*/

static void vErrorChecks( void *pvParameters )
{
  portTickType xDelayPeriod = mainNO_ERROR_FLASH_PERIOD;
  portTickType xLastWakeTime;

  /* The parameters are not used. */
  ( void ) pvParameters;

  /* Initialise xLastWakeTime to ensure the first call to vTaskDelayUntil()
  functions correctly. */
  xLastWakeTime = xTaskGetTickCount();

  /* Cycle for ever, delaying then checking all the other tasks are still
  operating without error.  If an error is detected then the delay period
  is decreased from mainNO_ERROR_FLASH_PERIOD to mainERROR_FLASH_PERIOD so
  the Check LED flash rate will increase. */
  for ( ;; )
    {
    /* Delay until it is time to execute again.  The delay period is
    shorter following an error. */
    vTaskDelayUntil( &xLastWakeTime, xDelayPeriod );

    /* Check all the standard demo application tasks are executing without
    error.  */
    if ( prvCheckOtherTasksAreStillRunning() != pdPASS )
      {
      /* An error has been detected in one of the tasks - flash faster. */
      xDelayPeriod = mainERROR_FLASH_PERIOD;
      }

    vParTestToggleLED( mainCHECK_LED );
    }
}
/*-----------------------------------------------------------*/

static portLONG prvCheckOtherTasksAreStillRunning( void )
{
  portLONG lReturn = ( portLONG ) pdPASS;

  /* Check all the demo tasks (other than the flash tasks) to ensure
  that they are all still running, and that none of them have detected
  an error. */

  if ( xArePollingQueuesStillRunning() != pdTRUE )
    {
    lReturn = ( portLONG ) pdFAIL;
    }

  if ( xAreSemaphoreTasksStillRunning() != pdTRUE )
    {
    lReturn = ( portLONG ) pdFAIL;
    }

  if ( xAreIntegerMathsTaskStillRunning() != pdTRUE )
    {
    lReturn = ( portLONG ) pdFAIL;
    }

  if ( xAreBlockingQueuesStillRunning() != pdTRUE )
    {
    lReturn = ( portLONG ) pdFAIL;
    }

  return lReturn;
}
