/*
	FreeRTOS.org V4.7.0 - Copyright (C) 2003-2007 Richard Barry.
	TriCore port - written by Adriaan Schmidt, Copyright (C) 2008 Siemens AG.

	This file is part of the FreeRTOS.org distribution.

	FreeRTOS.org is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
	The TriCore port is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	version 2 as published by the Free Software Foundation.

	FreeRTOS.org is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with FreeRTOS.org; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	A special exception to the GPL can be applied should you wish to distribute
	a combined work that includes FreeRTOS.org, without being obliged to provide
	the source code for any proprietary components.  See the licensing section
	of http://www.FreeRTOS.org for full details of how and when the exception
	can be applied.

	***************************************************************************
	See http://www.FreeRTOS.org for documentation, latest information, license
	and contact details.  Please ensure to read the configuration and relevant
	port sections of the online documentation.

	Also see http://www.SafeRTOS.com a version that has been certified for use
	in safety critical systems, plus commercial licensing, development and
	support options.
	***************************************************************************
*/

/* TriCore includes */
#include <machine/intrinsics.h>
#include <machine/wdtcon.h>
#include <tc1796b/cps.h>
#include <tc1796b/stm.h>

/* for interrupts */
#define pdTaskSwitchIrqPrio		1
#define pdTickIrqPrio			2
#define pdMaxIrqPrio			255
#include "port_irq.h"

/* FreeRTOS includes */
#include "FreeRTOS.h"
#include "task.h"

/*
 * These functions implement the portable.h interface
 */


/* ------------------------ Static variables ------------------------------ */
typedef struct {
	unsigned portLONG xPCXI;
	unsigned portLONG ulCritNest;
} portTaskInfo;

typedef portTaskInfo * tskTCB;

/* We require the address of the pxCurrentTCB variable, but we will
   interpret the TCB as a pointer to out own TaskInfo structure */
extern volatile tskTCB * volatile pxCurrentTCB;

static volatile tskTCB * volatile pxPreviousTCB = NULL;
static unsigned portBASE_TYPE prvTaskSwitchRequired = 0;

static unsigned portLONG prvTickReloadValue;

/* ------------------------ Global variables ------------------------------ */
volatile unsigned portLONG ulCriticalNesting = portINITIAL_CRITICAL_NESTING;

/*------------------------- macros ----------------------------------*/
/*
 * On Tricore all context save- and restore-operations
 * are handled implicitly by the hardware and compiler.
 * So in SAVE_CONTEXT we just keep a pointer to the last
 * task, vPortTaskSwitchIRQ will do the actual saving.
 * All RESTORE_CONTEXT does is invoking the task switch
 * interrupt of priority 1. This is the way the manual
 * suggests that context switches should be implemented.
 */
#define portSAVE_CONTEXT() {      \
	pxPreviousTCB = pxCurrentTCB; \
}

#define portRESTORE_CONTEXT() {   \
	CPU_SRC0.bits.SETR = 1;       \
	portENABLE_INTERRUPTS();      \
}

/* macro to convert a cx register (pcx, fcx, lcx) into a pointer */
#define CX2PTR( cx )                  \
	( ( ( cx & 0x000f0000 ) << 12 ) | \
	  ( ( cx & 0x0000ffff ) <<	6 ) )


/* ------------------------ function prototypes -------------------------- */
static void prvSetupTaskSwitchInterrupt( void );
static void prvSetupTimerInterrupt( void );

static portIRQ_FUNCTION_PROTO( vPortTaskSwitchIRQ );
static portIRQ_FUNCTION_PROTO( vPortTickIRQ );

#if ( configUSE_PREEMPTION == 1 )
static void vPortYieldFromTick( void );
#endif

/*------------------------- functions ----------------------------------*/

/*
 * Setup the task switch interrupt
 * the actual interrupt is generated by software using
 * the TriCore CPU's Service Request Node 0
 */
static void prvSetupTaskSwitchInterrupt( void )
{
	vIrqSetVector( pdTaskSwitchIrqPrio, vPortTaskSwitchIRQ, pdMaxIrqPrio );

	CPU_SRCm_t_nonv xSRC;
	xSRC.reg = 0;
	xSRC.bits.SRPN = pdTaskSwitchIrqPrio;
	xSRC.bits.SRE = 1;
	CPU_SRC0 = xSRC;
}

/*
 * Setup system timer to generate the tick interrupt
 */
static void prvSetupTimerInterrupt( void )
{
	vIrqSetVector( pdTickIrqPrio, vPortTickIRQ, pdTickIrqPrio );

	prvTickReloadValue = configCPU_CLOCK_HZ / configTICK_RATE_HZ;

	unlock_wdtcon();
	STM_CLC.bits.RMC = 1;
	lock_wdtcon();

	/* CMP0 for ticks */
	STM_CMP0.reg = STM_TIM0.reg + prvTickReloadValue;
	STM_CMCON.bits.MSIZE0 = 31;

	STM_ICR.reg |= STM_ICR_CMP0EN_MASK;

	STM_SRC0.bits.SRPN = pdTickIrqPrio;
	STM_SRC0.bits.SRE = 1;
}
/*-----------------------------------------------------------*/

static portIRQ_FUNCTION( vPortTaskSwitchIRQ )
{
	if ( pxPreviousTCB != NULL ) {
		(*pxPreviousTCB)->xPCXI = _mfcr( $pcxi );
		(*pxPreviousTCB)->ulCritNest = ulCriticalNesting;
	}

	if ( prvTaskSwitchRequired ) {
		vTaskSwitchContext();
	}

	ulCriticalNesting = (*pxCurrentTCB)->ulCritNest;
	_mtcr( $pcxi, (*pxCurrentTCB)->xPCXI );
	_isync();
	_dsync();

	/*
	 * this code is within an ISR, so obviously interrupts
	 * are enabled at this point. but in case our user
	 * task was executing a critical section, we need to disable
	 * interrupts again by patching the lower context (which is
	 * still in the CSA in memory, and will be restored when
	 * this ISR returns)
	 */
	if ( ulCriticalNesting > portNO_CRITICAL_NESTING ) {
		unsigned portLONG *pxCsaPtr =
                (unsigned portLONG *) CX2PTR( (*pxCurrentTCB)->xPCXI );
		pxCsaPtr[0] &= ~0x00800000;
	}

	prvTaskSwitchRequired = 0;
}

static portIRQ_FUNCTION( vPortTickIRQ )
{
	STM_CMP0.reg = STM_TIM0.reg + prvTickReloadValue;

#if ( configUSE_PREEMPTION == 1 )
	vPortYieldFromTick();
#else
	vTaskIncrementTick();
#endif
}

void vPortTaskReturnHandler( void )
{
#if ( INCLUDE_vTaskDelete == 1 )
	vTaskDelete( NULL );
#else
	for ( ;; ) vPortYield();
#endif
}

/*
 * See header file for description.
 */
portSTACK_TYPE *pxPortInitialiseStack(
		portSTACK_TYPE *pxTopOfStack,
		pdTASK_CODE pxCode,
		void *pvParameters )
{
	/*
	 * in this port the stack is not that important, as contexts
	 * are saved in context save areas...
	 * so we mainly setup these CSAs here
	 */
	unsigned portLONG xSavePCX, xSaveLR;
	unsigned portLONG xTempPCX, *pxCsaPtr;

	portTaskInfo *pxInfo = (portTaskInfo *)( pxTopOfStack - 1 );

	portENTER_CRITICAL();

	/* save registers */
	xSavePCX = _mfcr( $pcxi );
	__asm__( "mov.d %0, %%a11":"=r"( xSaveLR ) );

	_mtcr( $pcxi, 0 );
	_dsync();

	/* setup task return handler */
	__asm__( "call to_save_upper_context1" );
	__asm__( "to_save_upper_context1:" );
	__asm__( "mov.a %%a11, %0"::"r"( vPortTaskReturnHandler ) );

	/* setup initial task context */
	__asm__( "call to_save_upper_context2" );
	__asm__( "to_save_upper_context2:" );

	xTempPCX = _mfcr( $pcxi );
	_dsync();

	pxCsaPtr = (unsigned portLONG *) CX2PTR( xTempPCX );
	pxCsaPtr[1] = 0x000008ff;                               /* PSW */
	pxCsaPtr[2] = (unsigned portLONG) ( pxTopOfStack - 2 ); /* SP  */
	__asm__( "mov.a %%a11, %0"::"r"( pxCode ) );

	/* now save lower context */
	_svlcx();
	_isync();
	_dsync();

	/* and insert the parameter into a4 */
	xTempPCX = _mfcr( $pcxi );
	_dsync();
	pxCsaPtr = (unsigned portLONG *) CX2PTR( xTempPCX );
	pxCsaPtr[0] |= 0x00800000; /* enable interrupts */
	pxCsaPtr[8] = (unsigned portLONG) pvParameters;

	pxInfo->xPCXI = xTempPCX;
	pxInfo->ulCritNest = portNO_CRITICAL_NESTING;

	/* restore saved registers */
	__asm__( "mov.a %%a11, %0"::"r"( xSaveLR ) );
	_mtcr( $pcxi, xSavePCX );
	_dsync();

	portEXIT_CRITICAL();

	return (portSTACK_TYPE *) pxInfo;
}
/*-----------------------------------------------------------*/

void vPortCleanupTCB( tskTCB *pxTCB )
{
	unsigned portLONG xTempPCX;

	portENTER_CRITICAL();

	xTempPCX = (*pxTCB)->xPCXI;
	/* return the task's CSAs to the CPU's free CSA list */
	while ( xTempPCX != 0 ) {
		unsigned portLONG *pxCsaPtr =
                (unsigned portLONG *) CX2PTR( xTempPCX );
		unsigned portLONG xNextPCX = pxCsaPtr[0];
		pxCsaPtr[0] = _mfcr( $fcx );
		_mtcr( $fcx, xTempPCX );
		_dsync();
		xTempPCX = xNextPCX;
	}

	portEXIT_CRITICAL();
}


portBASE_TYPE xPortStartScheduler( void )
{
	/* Prepare the TriCore way of context switching. */
	prvSetupTaskSwitchInterrupt();

	/* Setup the hardware to generate the tick. */
	prvSetupTimerInterrupt();

	/* Restore the context of the first task that is going to run. */
	portRESTORE_CONTEXT();

	/*
	 * at this point the task switch interrupt takes over and switches
	 * the context to the first user task.
	 * we will never return here...
	 */

	return pdTRUE;
}
/*-----------------------------------------------------------*/

void vPortEndScheduler( void )
{
	/* this port is unstoppable */
}
/*-----------------------------------------------------------*/

/*
 * Context switch.
 * For this we have two functions: one to be called from
 * a user task, and one to be called from the tick ISR
 */
void vPortYield( void )
{
	portSAVE_CONTEXT();
	prvTaskSwitchRequired = 1;
	portRESTORE_CONTEXT();
}

#if ( configUSE_PREEMPTION == 1 )
void vPortYieldFromTick( void )
{
	portSAVE_CONTEXT();
	vTaskIncrementTick();
	prvTaskSwitchRequired = 1;
	portRESTORE_CONTEXT();
}
#endif

/*-----------------------------------------------------------*/
